@@ -1,17 +1,19 @@
+"""Metrics to record scores while training."""
 from dataclasses import dataclass
+from typing import Any, Dict
 
-import numpy as np
-
-from ivory.core.dict import Dict
+import ivory.core.collections
+from ivory.core.run import Run
 from ivory.core.state import State
 
 
 @dataclass
-class Metrics(Dict, State):
+class Metrics(ivory.core.collections.Dict, State):
+    """Metrics object."""
+
     def __post_init__(self):
         super().__post_init__()
-        self.epoch = 0
-        self.history = {}
+        self.history = ivory.core.collections.Dict()
 
     def __str__(self):
         metrics = []
@@ -24,28 +26,13 @@ class Metrics(Dict, State):
         args = str(self).replace(" ", ", ")
         return f"{class_name}({args})"
 
-    def on_epoch_start(self, run):
-        self.epoch = run.trainer.epoch
+    def on_epoch_begin(self, run: Run):
+        if run.trainer:
+            self.epoch = run.trainer.epoch
+        else:
+            self.epoch = 0
 
-    def on_train_start(self, run):
-        self.losses = []
-
-    def on_val_start(self, run):
-        self.losses = []
-
-    def step(self, output, target):
-        """Adds float loss to ``self.loss`` and returns a loss object (ex. tensor)."""
-        pass
-
-    def on_train_end(self, run):
-        if self.losses:
-            self["loss"] = np.mean(self.losses)
-
-    def on_val_end(self, run):
-        if self.losses:
-            self["val_loss"] = np.mean(self.losses)
-
-    def on_epoch_end(self, run):
+    def on_epoch_end(self, run: Run):
         self.update(self.metrics_dict(run))
         self.update_history()
 
@@ -56,6 +43,6 @@ class Metrics(Dict, State):
             else:
                 self.history[metric][self.epoch] = value
 
-    def metrics_dict(self, run):
-        """Returns an extra custom metrics dictionay."""
+    def metrics_dict(self, run: Run) -> Dict[str, Any]:
+        """Returns an extra custom metrics dictionary."""
         return {}
