@@ -2,12 +2,15 @@ import os
 import tempfile
 import time
 from dataclasses import dataclass
+from typing import Any, Dict
 
 import mlflow
 import yaml
 from mlflow.entities import Metric, Param
+from mlflow.utils.mlflow_tags import MLFLOW_PARENT_RUN_ID
 
 from ivory import utils
+from ivory.core.run import Run
 
 
 @dataclass
@@ -17,59 +20,67 @@ class Tracking:
     def __post_init__(self):
         self.client = mlflow.tracking.MlflowClient(self.tracking_uri)
 
-    def on_fit_start(self, run):
-        with tempfile.TemporaryDirectory() as tmpdir:
-            path = os.path.join(tmpdir, "params.yaml")
-            with open(path, "w") as file:
-                yaml.dump(run.params, file, sort_keys=False)
-            with utils.chdir(run.source_name):
-                self.client.log_artifacts(run.id, tmpdir)
-
-    def on_epoch_end(self, run):
+    def on_epoch_end(self, run: Run):
+        self.save_run(run, "current")
+        if not run.metrics:
+            return
         metrics = run.metrics.copy()
         monitor = run.monitor
         if monitor:
             metrics.update(best_score=monitor.best_score, best_epoch=monitor.best_epoch)
         self.log_metrics(run.id, metrics, run.metrics.epoch)
-        self.save_run(run, "current")
 
-    def on_fit_end(self, run):
-        self.client.set_terminated(run.id)
+    def on_fit_end(self, run: Run):
+        if self.client.get_run(run.id).info.status == "RUNNING":
+            self.client.set_terminated(run.id)
 
-    def on_test_end(self, run):
+    def on_test_end(self, run: Run):
         self.save_run(run, "test")
-        self.client.set_terminated(run.id)
 
-    def save_run(self, run, mode):
+    def log_params_artifact(self, run: Run):
+        with tempfile.TemporaryDirectory() as tmpdir:
+            path = os.path.join(tmpdir, "params.yaml")
+            with open(path, "w") as file:
+                yaml.dump(run.params, file, sort_keys=False)
+            with utils.path.chdir(run.source_name):
+                self.client.log_artifacts(run.id, tmpdir)
+
+    def log_params(self, run_id: str, params: Dict[str, Any]):
+        params_list = []
+        for key, value in params.items():
+            params_list.append(Param(key, to_str(value)))
+        self.client.log_batch(run_id, metrics=[], params=params_list, tags=[])
+
+    def log_metrics(self, run_id: str, metrics: Dict[str, float], step: int = 0):
+        ts = int(time.time() * 1000)  # timestamp in milliseconds.
+        metrics_ = [Metric(key, value, ts, step) for key, value in metrics.items()]
+        self.client.log_batch(run_id, metrics=metrics_, params=[], tags=[])
+
+    def save_run(self, run: Run, mode: str):
         with tempfile.TemporaryDirectory() as tmpdir:
             directory = os.path.join(tmpdir, mode)
             os.mkdir(directory)
             run.save(directory)
-            with utils.chdir(run.source_name):
+            with utils.path.chdir(run.source_name):
                 self.client.log_artifacts(run.id, tmpdir)
                 if mode != "current":
                     return
-                if run.monitor and run.monitor.is_best:
+                if run.monitor and run.monitor.is_best and run.monitor.best_epoch > -1:
                     os.rename(directory, directory.replace("current", "best"))
                     self.client.log_artifacts(run.id, tmpdir)
 
-    def log_params(self, run_id, params):
-        params_list = []
-        for key, value in params.items():
-            value = str(value)[:250]
-            params_list.append(Param(key, value))
-        self.client.log_batch(run_id, metrics=[], params=params_list, tags=[])
-
-    def log_metrics(self, run_id, metrics, step=0):
-        ts = int(time.time() * 1000)  # timestamp in milliseconds.
-        metrics = [Metric(key, value, ts, step) for key, value in metrics.items()]
-        self.client.log_batch(run_id, metrics=metrics, params=[], tags=[])
-
-    def set_tags(self, run_id, tags):
+    def set_tags(self, run_id: str, tags: Dict[str, Any]):
         for key, value in tags.items():
-            self.client.set_tag(run_id, key, value)
+            self.client.set_tag(run_id, key, to_str(value))
 
-    def create_tracker(self):
-        from ivory.core.tracker import Tracker
+    def set_parent_run_id(self, run_id: str, parent_run_id: str):
+        self.client.set_tag(run_id, MLFLOW_PARENT_RUN_ID, parent_run_id)
 
-        return Tracker(self.tracking_uri)
+
+def to_str(value: Any) -> str:
+    if isinstance(value, (list, tuple)):
+        return "[" + ", ".join(to_str(x) for x in value) + "]"
+    elif isinstance(value, float):
+        return f"{value:.4g}"
+    else:
+        return str(value)
