@@ -3,7 +3,7 @@ from dataclasses import dataclass
 import torch
 from torch.optim.lr_scheduler import ReduceLROnPlateau
 
-import ivory.callbacks.trainer
+import ivory.core.trainer
 from ivory.torch import utils
 
 try:
@@ -13,12 +13,13 @@ except ImportError:
 
 
 @dataclass
-class Trainer(ivory.callbacks.trainer.Trainer):
+class Trainer(ivory.core.trainer.Trainer):
     gpu: bool = False
     precision: int = 32  # Full precision (32), half precision (16).
     amp_level: str = "O1"
+    scheduler_step_mode: str = "epoch"
 
-    def on_fit_start(self, run):
+    def on_fit_begin(self, run):
         if self.gpu:
             run.model.cuda()
             if self.precision == 16:
@@ -26,16 +27,17 @@ class Trainer(ivory.callbacks.trainer.Trainer):
                     run.model, run.optimizer, opt_level=self.amp_level
                 )
 
-    def on_train_start(self, run):
+    def on_train_begin(self, run):
         run.model.train()
 
-    def train_step(self, index, input, target, run):
+    def train_step(self, run, index, input, target):
         if self.gpu:
             input = utils.cuda(input)
             target = utils.cuda(target)
-        output = run.model(input)
-        run.results.step(index, output, target)
-        loss = run.metrics.step(output, target)
+        output = self.forward(run.model, input)
+        if run.results:
+            run.results.step(index, output, target)
+        loss = run.metrics.step(input, output, target)
         optimizer = run.optimizer
         optimizer.zero_grad()
         if self.gpu and self.precision == 16:
@@ -44,33 +46,40 @@ class Trainer(ivory.callbacks.trainer.Trainer):
         else:
             loss.backward()
         optimizer.step()
+        if run.sheduler and self.scheduler_step_mode == "batch":
+            run.scheduler.step()
 
-    def on_val_start(self, run):
+    def on_val_begin(self, run):
         run.model.eval()
 
     @torch.no_grad()
-    def val_step(self, index, input, target, run):
+    def val_step(self, run, index, input, target):
         if self.gpu:
             input = utils.cuda(input)
             target = utils.cuda(target)
-        output = run.model(input)
-        run.results.step(index, output, target)
-        run.metrics.step(output, target)
+        output = self.forward(run.model, input)
+        if run.results:
+            run.results.step(index, output, target)
+        run.metrics.step(input, output, target)
 
     def on_epoch_end(self, run):
-        if run.scheduler:
+        if run.scheduler and self.scheduler_step_mode == "epoch":
             if isinstance(run.scheduler, ReduceLROnPlateau):
                 run.scheduler.step(run.monitor.score)
             else:
                 run.scheduler.step()
 
-    def on_test_start(self, run):
-        self.on_fit_start(run)
+    def on_test_begin(self, run):
+        self.on_fit_begin(run)
         run.model.eval()
 
     @torch.no_grad()
-    def test_step(self, index, input, run):
+    def test_step(self, run, index, input, *target):
         if self.gpu:
             input = utils.cuda(input)
-        output = run.model(input)
-        run.results.step(index, output)
+        output = self.forward(run.model, input)
+        if run.results:
+            run.results.step(index, output, *target)
+
+    def forward(self, model, input):
+        return model(input)
