@@ -1,4 +1,5 @@
-from typing import Any, Dict
+import itertools
+from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple
 
 
 def update_dict(org: Dict[str, Any], update: Dict[str, Any]) -> None:
@@ -6,11 +7,13 @@ def update_dict(org: Dict[str, Any], update: Dict[str, Any]) -> None:
 
     Examples:
         >>> x = {"a": 1, "b": {"x": "abc", "y": 2, "z": [0, 1, 2]}}
-        >>> update_dict(x, {"b": {"z": [0]}, "b.x": "def"})
+        >>> update_dict(x, {"b": {"z": [0, 1]}, "b.x": "def"})
         >>> x
-        {'a': 1, 'b': {'x': 'def', 'y': 2, 'z': [0]}}
+        {'a': 1, 'b': {'x': 'def', 'y': 2, 'z': [0, 1]}}
+        >>> update_dict(x, {"b.z.1": 10, "b.x": "def"})
+        >>> x
+        {'a': 1, 'b': {'x': 'def', 'y': 2, 'z': [0, 10]}}
     """
-    update = dot_to_list(update)  # for optuna
     for key, value in update.items():
         x = org
         attrs = key.split(".")
@@ -18,41 +21,43 @@ def update_dict(org: Dict[str, Any], update: Dict[str, Any]) -> None:
             x = x[attr]
         k = attrs[-1]
         if k not in x:
+            if "0" <= k[0] <= "9":
+                k = int(k)  # type:ignore
             x[k] = value
         elif isinstance(x[k], str) and x[k].startswith("$"):
             x[k] = value
-        elif type(x[k]) is not type(value):
+        elif type(x[k]) is not type(value) and x[k] is not None:
             raise ValueError(f"different type: {type(x[k])} != {type(value)}")
         else:
-            if isinstance(value, dict):
+            if isinstance(x[k], dict):
                 x[k].update(value)
             else:
                 x[k] = value
 
 
-def dot_to_list(x: Dict[str, Any]) -> Dict[str, Any]:
+def colon_to_list(x: Dict[str, Any]) -> Dict[str, Any]:
     """Converts suffix integers into a list.
 
     Examples:
-        >>> x = {"a.0": 1, "a.1": 3, "b.x.0": 10, "b.x.1": 20}
-        >>> dot_to_list(x)
-        {'a': [1, 3], 'b.x': [10, 20]}
+        >>> x = {"a:0": 1, "a:1": 3, "b.x:0": 10, "b.x:1": 20, "c": 100}
+        >>> colon_to_list(x)
+        {'a': [1, 3], 'b.x': [10, 20], 'c': 100}
     """
     update: Dict[str, Any] = {}
     for key, value in x.items():
-        head, _, tail = key.rpartition(".")
-        if "0" <= tail <= "9":
-            index = int(tail)
-            if index == 0:
-                if head in update:
-                    raise KeyError(key)
-                update[head] = [value]
-            elif head not in update or len(update[head]) != index:
-                raise KeyError(key)
-            else:
-                update[head].append(value)
-        else:
+        if ":" not in key:
             update[key] = value
+            continue
+        key, digit = key.split(":")
+        index = int(digit)
+        if index == 0:
+            if key in update:
+                raise KeyError(key)
+            update[key] = [value]
+        elif key not in update or len(update[key]) != index:
+            raise KeyError(key)
+        else:
+            update[key].append(value)
     return update
 
 
@@ -133,6 +138,20 @@ def get_fullnames(params, name, prefix="", dict_allowed=False):
             yield from get_fullnames(params[key], name, prefix_, dict_allowed)
 
 
+def create_update(params, args: Optional[Dict[str, Any]] = None, **kwargs):
+    if args is None:
+        args = {}
+    args.update(kwargs)
+    args = colon_to_list(args)
+    args_ = {}
+    update = {}
+    for name, value in args.items():
+        for fullname in get_fullnames(params, name):
+            update[fullname] = value
+            args_[name] = value
+    return update, args_
+
+
 def get_value(params, name):
     """
     Examples:
@@ -151,14 +170,6 @@ def get_value(params, name):
         return dot_get(params, fullnames[0])
 
 
-def create_update(params, **kwargs):
-    update = {}
-    for name, value in kwargs.items():
-        for fullname in get_fullnames(params, name):
-            update[fullname] = value
-    return update
-
-
 def match(params, **query):
     """Returns if params match the query or not.
 
@@ -195,3 +206,80 @@ def match(params, **query):
         elif value != cond:
             return False
     return True
+
+
+def product(params: Dict[str, Iterable[Any]]) -> Iterator[Dict[str, Any]]:
+    """
+    Examples:
+        >>> params = {"a": [1, 2], "b": [3, 4]}
+        >>> list(product(params))
+        [{'a': 1, 'b': 3}, {'a': 1, 'b': 4}, {'a': 2, 'b': 3}, {'a': 2, 'b': 4}]
+    """
+    for values in itertools.product(*params.values()):
+        args = {}
+        for name, value in zip(params.keys(), values):
+            args[name] = value
+        yield args
+
+
+def to_str(params):
+    t = []
+    for key, value in params.items():
+        if key == "verbose":
+            continue
+        if isinstance(value, str):
+            t.append(f"{key}={value!r}")
+        else:
+            t.append(f"{key}={value:.4g}")
+    return " ".join(t)
+
+
+def split_params(
+    params: Optional[Dict[str, Iterable[Any]]] = None, **kwargs
+) -> Tuple[Dict[str, Iterable[Any]], Dict[str, Any]]:
+    """Returns (`params`, `base_params`).
+
+    `params` for variable parameters and `base_params` for fixed parameters.
+
+    Args:
+        params: Parameter dictionay.
+        **kwargs: Optional parameters
+
+    Examples:
+        >>> params = {"a": range(2), "b": 3}
+        >>> params, base_params = split_params(params, c='abc', d=range(3))
+        >>> params
+        {'a': range(0, 2), 'd': range(0, 3)}
+        >>> base_params
+        {'b': 3, 'c': 'abc'}
+    """
+    if params is None:
+        params = {}
+    if kwargs:
+        params.update(kwargs)
+    params = params.copy()
+    base_params: Dict[str, Any] = {}
+    params_list = {arg: to_list(values) for arg, values in params.items()}
+    for arg, values in params_list.items():
+        if len(values) == 1:
+            base_params[arg] = values[0]
+            del params[arg]
+    return params, base_params
+
+
+def to_list(x) -> List[Any]:
+    """
+    Examples:
+        >>> to_list(range(3))
+        [0, 1, 2]
+        >>> to_list(100)
+        [100]
+        >>> to_list('abc')
+        ['abc']
+    """
+    if isinstance(x, str):
+        return [x]
+    try:
+        return list(x)
+    except TypeError:
+        return [x]
